@@ -1,8 +1,8 @@
 # -*- coding: iso-8859-1 -*-
-"""  
+"""
     MoinMoin - Spelling Action
-     
-    Word adding based on code by Christian Bird <chris.bird@lineo.com> 
+
+    Word adding based on code by Christian Bird <chris.bird@lineo.com>
 
     This action checks for spelling errors in a page using one or several
     word lists.
@@ -15,9 +15,9 @@
     Additionally, all words on the page "LocalSpellingWords" are added to
     the list of valid words, if that page exists.
 
-    @copyright: 2001 by Richard Jones <richard@bizarsoftware.com.au>  
-    @copyright: 2001-2004 by Jrgen Hermann <jh@web.de>  
-    @license: GNU GPL, see COPYING for details.  
+    @copyright: 2001 Richard Jones <richard@bizarsoftware.com.au>,
+                2001-2004 Juergen Hermann <jh@web.de>
+    @license: GNU GPL, see COPYING for details.
 """
 
 import os, re, codecs
@@ -33,14 +33,14 @@ def _getWordsFiles(request):
     for basedir in (request.cfg.moinmoin_dir, request.cfg.data_dir):
         localdict = os.path.join(basedir, 'dict')
         if os.path.isdir(localdict):
-            candidates.extend(map(
-                lambda f, d=localdict: os.path.join(d, f), os.listdir(localdict)))
+            candidates.extend(
+                [os.path.join(localdict, fn) for fn in os.listdir(localdict)])
 
     # validate candidate list (leave out directories!)
     wordsfiles = []
-    for file in candidates:
-        if os.path.isfile(file) and os.access(file, os.F_OK | os.R_OK):
-            wordsfiles.append(file)
+    for f in candidates:
+        if os.path.isfile(f) and os.access(f, os.F_OK | os.R_OK):
+            wordsfiles.append(f)
 
     # return validated file list
     return wordsfiles
@@ -55,13 +55,13 @@ def _loadWordsFile(request, dict, filename):
     request.clock.start('spellread')
     try:
         try:
-            file = codecs.open(filename, 'rt', config.charset)
-            lines = file.readlines()
+            f = codecs.open(filename, 'rt', config.charset)
+            lines = f.readlines()
         except UnicodeError:
-            file = codecs.open(filename, 'rt', 'iso-8859-1')
-            lines = file.readlines()
+            f = codecs.open(filename, 'rt', 'iso-8859-1')
+            lines = f.readlines()
     finally:
-        file.close()
+        f.close()
     _loadWords(lines, dict)
     request.clock.stop('spellread')
 
@@ -85,7 +85,7 @@ def _loadDict(request):
     else:
         wordsfiles = _getWordsFiles(request)
         if dbhash:
-            wordsdict = dbhash.open(cachename, 'n', 0666 & config.umask)
+            wordsdict = dbhash.open(cachename, 'n')
         else:
             wordsdict = {}
 
@@ -99,17 +99,15 @@ def _loadDict(request):
 
 
 def _addLocalWords(request):
-    import types
     from MoinMoin.PageEditor import PageEditor
-
     # get the new words as a string (if any are marked at all)
     try:
-        newwords = request.form['newwords']
+        newwords = request.form.getlist('newwords')
     except KeyError:
         # no new words checked
         return
     newwords = u' '.join(newwords)
-    
+
     # get the page contents
     lsw_page = PageEditor(request, request.cfg.page_local_spelling_words)
     words = lsw_page.get_raw_body()
@@ -126,7 +124,7 @@ def checkSpelling(page, request, own_form=1):
     _ = request.getText
 
     # first check to see if we we're called with a "newwords" parameter
-    if request.form.has_key('button_newwords'):
+    if 'button_newwords' in request.form:
         _addLocalWords(request)
 
     # load words
@@ -153,26 +151,25 @@ def checkSpelling(page, request, own_form=1):
             return ""
         w_enc = word.encode(config.charset)
         wl_enc = word.lower().encode(config.charset)
-        if not (wordsdict.has_key(w_enc) or
-                wordsdict.has_key(wl_enc) or
-                localwords.has_key(w_enc) or
-                localwords.has_key(wl_enc) ):
+        if not (w_enc in wordsdict or wl_enc in wordsdict or
+                w_enc in localwords or wl_enc in localwords):
             if not num_re.match(word):
                 badwords[word] = 1
         return ""
 
     # do the checking
     for line in text.split('\n'):
-        if line == '' or line[0] == '#': continue
+        if line == '' or line[0] == '#':
+            continue
         word_re.sub(checkword, line)
 
     if badwords:
         badwords = badwords.keys()
-        badwords.sort(lambda x,y: cmp(x.lower(), y.lower()))
+        badwords.sort(lambda x, y: cmp(x.lower(), y.lower()))
 
         # build regex recognizing the bad words
         badwords_re = r'(^|(?<!\w))(%s)(?!\w)'
-        badwords_re = badwords_re % ("|".join(map(re.escape, badwords)),)
+        badwords_re = badwords_re % ("|".join([re.escape(bw) for bw in badwords]), )
         badwords_re = re.compile(badwords_re, re.UNICODE)
 
         lsw_msg = ''
@@ -190,12 +187,12 @@ def checkSpelling(page, request, own_form=1):
 
         # add a form containing the bad words
         if own_form:
-            msg = msg + ('<form method="post" action="">\n'
-                         '<input type="hidden" name="action" value="%s">\n') % action_name
-        
+            msg = msg + ('<form method="post" action="%s">\n'
+                         '<input type="hidden" name="action" value="%s">\n') % (request.href(page.page_name), action_name)
+
         checkbox = '<input type="checkbox" name="newwords" value="%(word)s">%(word)s&nbsp;&nbsp;'
         msg = msg + (
-            " ".join(map(lambda w, cb=checkbox: cb % {'word': wikiutil.escape(w),}, badwords)) +
+            " ".join([checkbox % {'word': wikiutil.escape(w, True), } for w in badwords]) +
             '<p><input type="submit" name="button_newwords" value="%s"></p>' %
                 _('Add checked words to dictionary')
         )
@@ -212,15 +209,22 @@ def checkSpelling(page, request, own_form=1):
 
 def execute(pagename, request):
     _ = request.getText
+
     page = Page(request, pagename)
+    if not request.user.may.write(request.cfg.page_local_spelling_words):
+        request.theme.add_msg(_("You can't save spelling words."), "error")
+        page.send_page()
+        return
+
     if request.user.may.read(pagename):
         badwords, badwords_re, msg = checkSpelling(page, request)
     else:
         badwords = []
-        msg = _("You can't check spelling on a page you can't read.")
+        request.theme.add_msg(_("You can't check spelling on a page you can't read."), "error")
 
+    request.theme.add_msg(msg, "dialog")
     if badwords:
-        page.send_page(request, msg=msg, hilite_re=badwords_re)
+        page.send_page(hilite_re=badwords_re)
     else:
-        page.send_page(request, msg=msg)
+        page.send_page()
 
