@@ -175,6 +175,7 @@ _ = lambda x: x
 class Parser:
     caching = 1
     Dependencies = Dependencies # copy dependencies from module-scope
+    extensions = ['.rst', '.rest', ]
     quickhelp = _("""\
 {{{
 Emphasis: *italic* **bold** ``monospace``
@@ -363,7 +364,7 @@ class MoinTranslator(html4css1.HTMLTranslator):
                 self.wiki_text = self.fixup_wiki_formatting(self.wiki_text)
                 self.add_wiki_markup()
 
-            if prefix == 'drawing': # twikidraw drawing
+            if prefix == 'drawing':
                 self.process_wiki_text("[[%s]]" % refuri)
                 self.wiki_text = self.fixup_wiki_formatting(self.wiki_text)
                 self.add_wiki_markup()
@@ -390,6 +391,9 @@ class MoinTranslator(html4css1.HTMLTranslator):
                 # for images with targets).
                 if not [i for i in node.children if i.__class__ == docutils.nodes.image]:
                     node['classes'].append('interwiki')
+            elif prefix == 'javascript':
+                # is someone trying to do XSS with javascript?
+                node['refuri'] = 'javascript:alert("it does not work")'
             elif prefix != '':
                 # Some link scheme (http, file, https, mailto, etc.), add class
                 # information if the reference doesn't have a child image (don't
